/* vi: set sw=4 ts=4: */
/*
 * Mini syslogd implementation for busybox
 *
 * Copyright (C) 1999,2000 by Lineo, inc. and Erik Andersen
 * Copyright (C) 1999,2000,2001 by Erik Andersen <andersee@debian.org>
 *
 * Copyright (C) 2000 by Karl M. Hegbloom <karlheg@debian.org>
 *
 * "circular buffer" Copyright (C) 2001 by Gennady Feldman <gfeldman@cachier.com>
 *
 * Maintainer: Gennady Feldman <gena01@cachier.com> as of Mar 12, 2001
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <netdb.h>
#include <paths.h>
#include <signal.h>
#include <stdarg.h>
#include <time.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/un.h>
#include <sys/param.h>
#include <sys/stat.h>
#include "busybox.h"

/* NEW function phototype --spark >> */
int check_file_size(char *,int,int,int*);
/* << end of NEW function phototype */

/* NEW definition & global var--spark >> */
#define FIREWALL_LOG_FILE	"/var/log/message.firewall"
#define UPNP_LOG_FILE		"/var/log/message.upnp"
#define CONN_LOG_FILE		"/var/log/message.connect"
#define TMP_LOG_FILE		"/var/log/message.tmp"
#define FIREWALL_LOG		0
#define UPNP_LOG		1
#define CONN_LOG		2
char logfilepath[128];
/* << end of NEW definition & global var */


/* SYSLOG_NAMES defined to pull some extra junk from syslog.h */
#define SYSLOG_NAMES
#include <sys/syslog.h>
#include <sys/uio.h>

/* Path for the file where all log messages are written */
#define __LOG_FILE "/var/log/messages"

/* Path to the unix socket */
static char lfile[BUFSIZ];

static char *logFilePath = __LOG_FILE;

/* interval between marks in seconds */
static int MarkInterval = 20 * 60;

/* localhost's name */
static char LocalHostName[32];

#ifdef BB_FEATURE_REMOTE_LOG
#include <netinet/in.h>
/* udp socket for logging to remote host */
static int remotefd = -1;
/* where do we log? */
static char *RemoteHost;
/* what port to log to? */
static int RemotePort = 514;
/* To remote log or not to remote log, that is the question. */
static int doRemoteLog = FALSE;
static int local_logging = FALSE;
#endif

//spark for share memory
#include <sys/ipc.h>
#include <sys/sem.h>
#include <sys/shm.h>

/* our shared key */
static const long FIRE_KEY_ID = 0x4000; /*"GENA"*/
static const long CONN_KEY_ID = 0x1000; /*"GENA"*/
static const long UPNP_KEY_ID = 0x500; /*"GENA"*/

int *fire_count;
int *conn_count;
int *upnp_count;
int mutex2;
static int fire_count_shmid = -1;
static int conn_count_shmid = -1;
static int upnp_count_shmid = -1;
void GBT_count_init();
void GBT_count_cleanup(void);

/* circular buffer variables/structures */
#ifdef BB_FEATURE_IPC_SYSLOG
#if __GNU_LIBRARY__ < 5
#error Sorry.  Looks like you are using libc5.
#error libc5 shm support isnt good enough.
#error Please disable BB_FEATURE_IPC_SYSLOG
#endif

#include <sys/ipc.h>
#include <sys/sem.h>
#include <sys/shm.h>


/* our shared key */
static const long KEY_ID = 0x414e4547; /*"GENA"*/

// Semaphore operation structures
static struct shbuf_ds {
	int size;		// size of data written
	int head;		// start of message list
	int tail;		// end of message list
	char data[1];		// data/messages
} *buf = NULL;			// shared memory pointer

static struct sembuf SMwup[1] = {{1, -1, IPC_NOWAIT}}; // set SMwup
static struct sembuf SMwdn[3] = {{0, 0}, {1, 0}, {1, +1}}; // set SMwdn

static int 	shmid = -1;	// ipc shared memory id
static int 	s_semid = -1;	// ipc semaphore id
int	data_size = 16000; // data size
int	shm_size = 16000 + sizeof(*buf); // our buffer size
static int circular_logging = FALSE;

/*
 * sem_up - up()'s a semaphore.
 */
static inline void sem_up(int semid)
{
	if ( semop(semid, SMwup, 1) == -1 )
		perror_msg_and_die("semop[SMwup]");
}

/*
 * sem_down - down()'s a semaphore
 */
static inline void sem_down(int semid)
{
	if ( semop(semid, SMwdn, 3) == -1 )
		perror_msg_and_die("semop[SMwdn]");
}


void ipcsyslog_cleanup(void){
	printf("Exiting Syslogd!\n");
	if (shmid != -1)
		shmdt(buf);

	if (shmid != -1)
		shmctl(shmid, IPC_RMID, NULL);
	if (s_semid != -1)
		semctl(s_semid, 0, IPC_RMID, 0);
}

void ipcsyslog_init(void){
	if (buf == NULL){
	    if ((shmid = shmget(KEY_ID, shm_size, IPC_CREAT | 1023)) == -1)
		    	perror_msg_and_die("shmget");


	    if ((buf = shmat(shmid, NULL, 0)) == NULL)
    			perror_msg_and_die("shmat");


	    buf->size=data_size;
	    buf->head=buf->tail=0;

	    // we'll trust the OS to set initial semval to 0 (let's hope)
	    if ((s_semid = semget(KEY_ID, 2, IPC_CREAT | IPC_EXCL | 1023)) == -1){
	    	if (errno == EEXIST){
		   if ((s_semid = semget(KEY_ID, 2, 0)) == -1)
		    perror_msg_and_die("semget");
		}else
    			perror_msg_and_die("semget");
	    }
	}else{
		printf("Buffer already allocated just grab the semaphore?");
	}
}

/* write message to buffer */
void circ_message(const char *msg){
	int l=strlen(msg)+1; /* count the whole message w/ '\0' included */

	sem_down(s_semid);

	/*
	 * Circular Buffer Algorithm:
	 * --------------------------
	 *
	 * Start-off w/ empty buffer of specific size SHM_SIZ
	 * Start filling it up w/ messages. I use '\0' as separator to break up messages.
	 * This is also very handy since we can do printf on message.
	 *
	 * Once the buffer is full we need to get rid of the first message in buffer and
	 * insert the new message. (Note: if the message being added is >1 message then
	 * we will need to "remove" >1 old message from the buffer). The way this is done
	 * is the following:
	 *	When we reach the end of the buffer we set a mark and start from the beginning.
	 *	Now what about the beginning and end of the buffer? Well we have the "head"
	 *	index/pointer which is the starting point for the messages and we have "tail"
	 *	index/pointer which is the ending point for the messages. When we "display" the
	 *	messages we start from the beginning and continue until we reach "tail". If we
	 *	reach end of buffer, then we just start from the beginning (offset 0). "head" and
	 *	"tail" are actually offsets from the beginning of the buffer.
	 *
	 * Note: This algorithm uses Linux IPC mechanism w/ shared memory and semaphores to provide
	 * 	 a threasafe way of handling shared memory operations.
	 */
	if ( (buf->tail + l) < buf->size ){
		/* before we append the message we need to check the HEAD so that we won't
		   overwrite any of the message that we still need and adjust HEAD to point
		   to the next message! */
		if ( buf->tail < buf->head){
			if ( (buf->tail + l) >= buf->head ){
			  /* we need to move the HEAD to point to the next message
			   * Theoretically we have enough room to add the whole message to the
			   * buffer, because of the first outer IF statement, so we don't have
			   * to worry about overflows here!
			   */
			   int k= buf->tail + l - buf->head; /* we need to know how many bytes
			   					we are overwriting to make
								enough room */
			   char *c=memchr(buf->data+buf->head + k,'\0',buf->size - (buf->head + k));
			   if (c != NULL) {/* do a sanity check just in case! */
			   	buf->head = c - buf->data + 1; /* we need to convert pointer to
								  offset + skip the '\0' since
								  we need to point to the beginning
								  of the next message */
				/* Note: HEAD is only used to "retrieve" messages, it's not used
					when writing messages into our buffer */
			   }else{ /* show an error message to know we messed up? */
			   	printf("Weird! Can't find the terminator token??? \n");
			   	buf->head=0;
			   }
			}
		} /* in other cases no overflows have been done yet, so we don't care! */

		/* we should be ok to append the message now */
		strncpy(buf->data + buf->tail,msg,l); /* append our message */
		buf->tail+=l; /* count full message w/ '\0' terminating char */
	}else{
		/* we need to break up the message and "circle" it around */
		char *c;
		int k=buf->tail + l - buf->size; /* count # of bytes we don't fit */
		
		/* We need to move HEAD! This is always the case since we are going
		 * to "circle" the message.
		 */
		c=memchr(buf->data + k ,'\0', buf->size - k);
		
		if (c != NULL) /* if we don't have '\0'??? weird!!! */{
			/* move head pointer*/
			buf->head=c-buf->data+1; 
			
			/* now write the first part of the message */			
			strncpy(buf->data + buf->tail, msg, l - k - 1);
			
			/* ALWAYS terminate end of buffer w/ '\0' */
			buf->data[buf->size-1]='\0'; 
			
			/* now write out the rest of the string to the beginning of the buffer */
			strcpy(buf->data, &msg[l-k-1]);

			/* we need to place the TAIL at the end of the message */
			buf->tail = k + 1;
		}else{
			printf("Weird! Can't find the terminator token from the beginning??? \n");
			buf->head = buf->tail = 0; /* reset buffer, since it's probably corrupted */
		}
		
	}
	sem_up(s_semid);
}
#endif
/* Note: There is also a function called "message()" in init.c */
/* Print a message to the log file. */
static void message (char *fmt, ...) __attribute__ ((format (printf, 1, 2)));
static void message (char *fmt, ...)
{
	int fd;
	struct flock fl;
	va_list arguments;

	//printf("<logfilepath:%s>\n",logfilepath);
	fl.l_whence = SEEK_SET;
	fl.l_start  = 0;
	fl.l_len    = 1;

#ifdef BB_FEATURE_IPC_SYSLOG
	if ((circular_logging == TRUE) && (buf != NULL)){
			char b[1024];
			va_start (arguments, fmt);
			vsprintf (b, fmt, arguments);
			va_end (arguments);
			circ_message(b);

	}else
#endif
//	if ((fd = device_open (logFilePath,
	if ((fd = device_open (logfilepath,
						   O_WRONLY | O_CREAT | O_NOCTTY | O_APPEND |
						   O_NONBLOCK)) >= 0) {
		fl.l_type = F_WRLCK;
		fcntl (fd, F_SETLKW, &fl);
		va_start (arguments, fmt);
		vdprintf (fd, fmt, arguments);
		va_end (arguments);
		fl.l_type = F_UNLCK;
		fcntl (fd, F_SETLKW, &fl);
		close (fd);
	} else {
		/* Always send console messages to /dev/console so people will see them. */
		if ((fd = device_open (_PATH_CONSOLE,
							   O_WRONLY | O_NOCTTY | O_NONBLOCK)) >= 0) {
			va_start (arguments, fmt);
			vdprintf (fd, fmt, arguments);
			va_end (arguments);
			close (fd);
		} else {
			fprintf (stderr, "Bummer, can't print: ");
			va_start (arguments, fmt);
			vfprintf (stderr, fmt, arguments);
			fflush (stderr);
			va_end (arguments);
		}
	}
}
static int logMessage (int pri, char *msg)
{
	time_t now;
	char *timestamp;
	static char res[24] = "";
	CODE *c_pri, *c_fac;

	if (pri != 0) {
		for (c_fac = facilitynames;
				c_fac->c_name && !(c_fac->c_val == LOG_FAC(pri) << 3); c_fac++);
		for (c_pri = prioritynames;
				c_pri->c_name && !(c_pri->c_val == LOG_PRI(pri)); c_pri++);
		if (c_fac->c_name == NULL || c_pri->c_name == NULL)
			snprintf(res, sizeof(res), "<%d>", pri);
		else
			snprintf(res, sizeof(res), "%s.%s", c_fac->c_name, c_pri->c_name);
	}

	if (strlen(msg) < 16 || msg[3] != ' ' || msg[6] != ' ' ||
			msg[9] != ':' || msg[12] != ':' || msg[15] != ' ') {
		time(&now);
		timestamp = ctime(&now) + 4;
		timestamp[15] = '\0';
	} else {
		timestamp = msg;
		timestamp[15] = '\0';
		msg += 16;
	}

	/* todo: supress duplicates */

#ifdef BB_FEATURE_REMOTE_LOG
	/* send message to remote logger */
	if ( -1 != remotefd){
        static const int IOV_COUNT = 2;
		struct iovec iov[IOV_COUNT];
		struct iovec *v = iov;

		memset(&res, 0, sizeof(res));
		snprintf(res, sizeof(res), "<%d>", pri);
		v->iov_base = res ;
		v->iov_len = strlen(res);          
		v++;

		v->iov_base = msg;
		v->iov_len = strlen(msg);          
writev_retry:
		if ( -1 == writev(remotefd,iov, IOV_COUNT)){
			if (errno == EINTR) goto writev_retry;
			error_msg_and_die("syslogd: cannot write to remote file handle on"
					"%s:%d",RemoteHost,RemotePort);
		}
	}
	if (local_logging == TRUE)
#endif
	/* now spew out the message to wherever it is supposed to go */
	//log raw time from 1970/01/01
	sprintf(timestamp,"%d", (int)time((time_t *)NULL) );
	
//    fprintf(stderr,"%s#%s#%s#%s\n", timestamp, LocalHostName, res, msg);

//	if (strstr(msg,"conn")==msg)
	if ((strstr(msg,"conn")==msg)&&(msg[7]==' ')&&(msg[9]==' '))
	{
		strcpy(logfilepath,CONN_LOG_FILE);
		GIGA_p(mutex2);
		*conn_count=*conn_count+1;
		GIGA_v(mutex2);
		check_file_size(CONN_LOG_FILE,500,50,conn_count);//1000,1
	}
	else if (strstr(msg,"upnp")==msg)
	{
	    	strcpy(logfilepath,UPNP_LOG_FILE);        
	    	GIGA_p(mutex2);
	    	*upnp_count=*upnp_count+1;
	    	GIGA_v(mutex2);
	    	check_file_size(UPNP_LOG_FILE,500,50,upnp_count);//500,1
	}
	else if ( strstr(msg,"klogd: GR=") )
	{
		strcpy(logfilepath,FIREWALL_LOG_FILE);        
//		    printf("count:GIGA_p:3\n");
	    	GIGA_p(mutex2);
	    	//printf("<%d><%d>",*fire_count,*fire_count+1);
//		    printf("count:GIGA_v:3\n");
	    	*fire_count=*fire_count+1;
	//    	printf("#<%d>#\n",*fire_count);
	    	GIGA_v(mutex2);
//		    printf("count:GIGA_e:3\n");
	    	check_file_size(FIREWALL_LOG_FILE,900,50,fire_count);//4000,1
	}
	else 
	{
	    	//strcpy(logfilepath,logFilePath);
	    	//check_file_size(logFilePath,10,5,);
		return 0;
	}
		
	message("%s %s\n", timestamp, msg);	
	return 0;
}

static void quit_signal(int sig)
{
	logMessage(LOG_SYSLOG | LOG_INFO, "System log daemon exiting.");
	unlink(lfile);
#ifdef BB_FEATURE_IPC_SYSLOG
	ipcsyslog_cleanup();
#endif
	GBT_count_cleanup();
	exit(TRUE);
}

static void domark(int sig)
{
	if (MarkInterval > 0) {
		logMessage(LOG_SYSLOG | LOG_INFO, "-- MARK --");
		alarm(MarkInterval);
	}
}

/* This must be a #define, since when DODEBUG and BUFFERS_GO_IN_BSS are
 * enabled, we otherwise get a "storage size isn't constant error. */
#define BUFSIZE 1023
#if 0
static int serveConnection (int conn)
{
	RESERVE_BB_BUFFER(tmpbuf, BUFSIZE + 1);
	int    n_read;
	char *p = tmpbuf;

	n_read = read (conn, tmpbuf, BUFSIZE );

	RELEASE_BB_BUFFER (tmpbuf);
	return n_read;
}
#else
static int serveConnection (int conn)
{
	RESERVE_BB_BUFFER(tmpbuf, BUFSIZE + 1);
	int    n_read=0;
	char *p = tmpbuf;
	
		int           pri = (LOG_USER | LOG_NOTICE);
		char          line[ BUFSIZE + 1 ]="";
		unsigned char c=0;
		char *q = line;

	n_read = read (conn, tmpbuf, BUFSIZE );

	while (p < tmpbuf + n_read) {

		pri = (LOG_USER | LOG_NOTICE);
		line[ BUFSIZE + 1 ]="";
		memset(&line, 0, sizeof(BUFSIZE + 1));
		c=0;
		*q = line;

		tmpbuf[ n_read - 1 ] = '\0';

		while (p && (c = *p) && q < &line[ sizeof (line) - 1 ]) {
			if (c == '<') {
			/* Parse the magic priority number. */
				pri = 0;
				while (isdigit (*(++p))) {
					pri = 10 * pri + (*p - '0');
				}
				if (pri & ~(LOG_FACMASK | LOG_PRIMASK)){
					pri = (LOG_USER | LOG_NOTICE);
				}
			} else if (c == '\n') {
				*q++ = ' ';
			} else if (iscntrl (c) && (c < 0177)) {
				*q++ = '^';
				*q++ = c ^ 0100;
			} else {
				*q++ = c;
			}
			p++;
		}
		*q = '\0';
		p++;
		/* Now log it */

		logMessage (pri, line);
	}
	RELEASE_BB_BUFFER (tmpbuf);
	return n_read;
}
#endif

#ifdef BB_FEATURE_REMOTE_LOG
static void init_RemoteLog (void){

  struct sockaddr_in remoteaddr;
  struct hostent *hostinfo;
  int len = sizeof(remoteaddr);

  memset(&remoteaddr, 0, len);

  remotefd = socket(AF_INET, SOCK_DGRAM, 0);

  if (remotefd < 0) {
    error_msg_and_die("syslogd: cannot create socket");
  }

  hostinfo = xgethostbyname(RemoteHost);

  remoteaddr.sin_family = AF_INET;
  remoteaddr.sin_addr = *(struct in_addr *) *hostinfo->h_addr_list;
  remoteaddr.sin_port = htons(RemotePort);

  /* 
     Since we are using UDP sockets, connect just sets the default host and port 
     for future operations
  */
  if ( 0 != (connect(remotefd, (struct sockaddr *) &remoteaddr, len))){
    error_msg_and_die("syslogd: cannot connect to remote host %s:%d", RemoteHost, RemotePort);
  }

}
#endif

static void doSyslogd (void) __attribute__ ((noreturn));
static void doSyslogd (void)
{
	struct sockaddr_un sunx;
	socklen_t addrLength;
	
	int sock_fd;
	fd_set fds;
	fd_set readfds;
		
	//spark for select no block
	struct timeval timeout;
		int    n_ready=0;
		int    fd=0;	
					int   conn=0;
//	int i=0;
	/* Set up signal handlers. */
	signal (SIGINT,  quit_signal);
	signal (SIGTERM, quit_signal);
	signal (SIGQUIT, quit_signal);
	signal (SIGHUP,  SIG_IGN);
	signal (SIGCHLD,  SIG_IGN);
#ifdef SIGCLD
	signal (SIGCLD,  SIG_IGN);
#endif
	signal (SIGALRM, domark);
	alarm (MarkInterval);

	/* Create the syslog file so realpath() can work. */
	if (realpath (_PATH_LOG, lfile) != NULL)
		unlink (lfile);

	memset (&sunx, 0, sizeof (sunx));
	sunx.sun_family = AF_UNIX;
	strncpy (sunx.sun_path, lfile, sizeof (sunx.sun_path));
	if ((sock_fd = socket (AF_UNIX, SOCK_STREAM, 0)) < 0)
		perror_msg_and_die ("Couldn't get file descriptor for socket " _PATH_LOG);

	addrLength = sizeof (sunx.sun_family) + strlen (sunx.sun_path);
	if ((bind (sock_fd, (struct sockaddr *) &sunx, addrLength)) || (listen (sock_fd, 5)))
		perror_msg_and_die ("Could not connect to socket " _PATH_LOG);

	if (chmod (lfile, 0666) < 0)
		perror_msg_and_die ("Could not set permission on " _PATH_LOG);

	FD_ZERO (&fds);
	FD_SET (sock_fd, &fds);

#ifdef BB_FEATURE_IPC_SYSLOG
	if (circular_logging == TRUE ){
	   ipcsyslog_init();
	}
#endif
//spark
	GBT_count_init();
        if ((mutex2 = GIGA_initsem(SEMID_LOG_VIEW)) < 0)
            {
       		fprintf( stderr, "syslogd: GIGA_initsam fail\n");
            	//return -1;
            }        
	
        #ifdef BB_FEATURE_REMOTE_LOG
        if (doRemoteLog == TRUE){
          init_RemoteLog();
        }
        #endif
	
	
	//open message.firewall,message.connect,message.upnp file
#if 1
	{
	   FILE *log_fp;
	   log_fp=fopen(FIREWALL_LOG_FILE,"w");
	   fclose(log_fp);
	   
	   log_fp=fopen(UPNP_LOG_FILE,"w");
	   fclose(log_fp);
	   
	   log_fp=fopen(CONN_LOG_FILE,"w");
	   fclose(log_fp);
	}
#endif
	logMessage (LOG_SYSLOG | LOG_INFO, "syslogd started: " BB_BANNER);
	logMessage (LOG_USER | LOG_INFO,"conn: 3 0 0");
	for (;;) {

//		fd_set readfds;
		n_ready=0;
		fd=0;
		
		// by jkglee, 2003.01.13, move to logMessage() ...
		//spark for check_file_size;
		//check_file_size(FIREWALL_LOG_FILE,4000,2000);
		//check_file_size(CONN_LOG_FILE,1000,500);
		//check_file_size(UPNP_LOG_FILE,500,250);
		//check_file_size(logFilePath,10,5);
		
		memcpy (&readfds, &fds, sizeof (fds));

#if 0       
        /* by jkglee, integrate proc_keeper, time interva
		 * l change from 5000 to 75 seconds */
        timeout.tv_sec = 75;
        timeout.tv_usec = 0;
#else		
		//spark for select timeval
		timeout.tv_sec = 0;
		timeout.tv_usec = 5000000;
#endif		
		
		if ((n_ready = select (FD_SETSIZE, &readfds, NULL, NULL, &timeout)) < 0) {
//			printf("select\n");
//		if ((n_ready = select (FD_SETSIZE, &readfds, NULL, NULL, NULL)) < 0) {
			if (errno == EINTR) continue; /* alarm may have happened. */
			perror_msg_and_die ("select error");
		}

		for (fd = 0; (n_ready > 0) && (fd < FD_SETSIZE); fd++) {
			
			if (FD_ISSET (fd, &readfds)) {
//			printf("Check fd is set? yes it is!\n");
				--n_ready;

				if (fd == sock_fd) {
					conn=0;

//					printf("New Connection request.\n");
					if ((conn = accept (sock_fd, (struct sockaddr *) &sunx, &addrLength)) < 0) {
						perror_msg_and_die ("accept error");
					}

					FD_SET(conn, &fds);
//					printf("conn: %i, set_size: %i\n",conn,FD_SETSIZE);
			  	} else {
//		 			printf("Serving connection: %i,count:%d\n",fd,i);
//			        i++;
//					printf("Sockfd: %i\n",sock_fd);
					  if ( serveConnection(fd) <= 0 ) {
					    close (fd);
					    FD_CLR(fd, &fds);
            				  }
				} /* fd == sock_fd */
			}/* FD_ISSET() */
		}/* for */
	
	} /* for main loop */
}

extern int syslogd_main(int argc, char **argv)
{
	int opt;
	int doFork = TRUE;

	char *p;

	/* do normal option parsing */
	while ((opt = getopt(argc, argv, "m:nO:R:LC")) > 0) {
		switch (opt) {
			case 'm':
				MarkInterval = atoi(optarg) * 60;
				break;
			case 'n':
				doFork = FALSE;
				break;
			case 'O':
				logFilePath = xstrdup(optarg);
				break;
#ifdef BB_FEATURE_REMOTE_LOG
			case 'R':
				RemoteHost = xstrdup(optarg);
				if ( (p = strchr(RemoteHost, ':'))){
					RemotePort = atoi(p+1);
					*p = '\0';
				}
				doRemoteLog = TRUE;
				break;
			case 'L':
				local_logging = TRUE;
				break;
#endif
#ifdef BB_FEATURE_IPC_SYSLOG
			case 'C':
				circular_logging = TRUE;
				break;
#endif
			default:
				show_usage();
		}
	}

#ifdef BB_FEATURE_REMOTE_LOG
	/* If they have not specified remote logging, then log locally */
	if (doRemoteLog == FALSE)
		local_logging = TRUE;
#endif


	/* Store away localhost's name before the fork */
	gethostname(LocalHostName, sizeof(LocalHostName));
	if ((p = strchr(LocalHostName, '.'))) {
		*p++ = '\0';
	}

	umask(0);

	if (doFork == TRUE) {
#if !defined(__UCLIBC__) || defined(__UCLIBC_HAS_MMU__)
		if (daemon(0, 1) < 0)
			perror_msg_and_die("daemon");
#else
			error_msg_and_die("daemon not supported");
#endif
	}
	
        // by jkglee, 2002.11.28, for firmware upgrade ...
        {
            FILE *fpFile = NULL;
            if ( (fpFile = fopen("/var/syslogd.pid", "w")) != NULL )
            {
                fprintf(fpFile, "%d", getpid());
                fclose(fpFile);
            }
        }
	
	doSyslogd();

	return EXIT_SUCCESS;
}

/*
Local Variables
c-file-style: "linux"
c-basic-offset: 4
tab-width: 4
End:
*/
/* kernel(firewall) and userspace(connection &upnp) log >> */

//#define DBG

#ifdef	DBG
#define GBT_ENTER() printf("(enter) %s, %s line %i\n", __FUNCTION__,__FILE__,__LINE__)
#define GBT_EXIT() printf("(exit) %s, %s line %i\n", __FUNCTION__,__FILE__,__LINE__)
#define G_DBSTR(number, str, val) printf("Spark : %s, %s, %d, \"%s\" = %d\n", __FILE__, __FUNCTION__, number, str, val)
#else
#define GBT_ENTER()
#define GBT_EXIT() 
#define G_DBSTR(number, str, val)
#endif

#define STRBUFSIZE		512
int check_file_size(char *file,int line_max,int line_del,int *line_current)
{
 	char strbuf[STRBUFSIZE];
 	FILE *srcfp,*dstfp;
 	int i;
//	char movebuf[1024]="";
	char movebuf[512]="";
	int n_read=0;
 	GBT_ENTER();

	memset(&strbuf, 0, sizeof(STRBUFSIZE));
	memset(&movebuf, 0, sizeof(STRBUFSIZE));
	//printf("check_file_size:<line_current:%d>\n",*line_current);
	if(*line_current<line_max)
		return 0;
	
	if( (srcfp=fopen(file,"r")) ==NULL)
	    {
		printf("syslogd:open %s error\n",file);	
	   	return -1;
	    }
	if( (dstfp=fopen(TMP_LOG_FILE,"w"))== NULL)
    	    {
	    	printf("syslogd:open %s error\n",TMP_LOG_FILE);	
	    	//spark 20030220
	    	fclose(srcfp);
	    	return -1;
	    }
	for(i=0;i<line_del;i++)
	    {
	    	fgets(strbuf,STRBUFSIZE,srcfp);
	    }
//    printf("syslog:cheak_file_size_GIGA_p:1\n");	
	GIGA_p(mutex2);
//    printf("syslog:cheak_file_size_GIGA_v:1\n");	
	*line_current=*line_current-line_del;
	GIGA_v(mutex2);
//    printf("syslog:cheak_file_size_GIGA_e:1\n");	
	do{
	    n_read=fread(movebuf,1,1024,srcfp); 
	    fwrite(movebuf,1,n_read,dstfp);
	    }while(n_read>0);
	
	fclose(dstfp);
	fclose(srcfp);
//	sprintf(strbuf,"mv %s %s",TMP_LOG_FILE,file);		
        //spark 20030224--move semaphere to here.
        {
        
//    printf("syslog:cheak_file_size_GIGA_p:2:mutex2:%d\n",mutex2);	
	GIGA_p(mutex2);
//    printf("syslog:cheak_file_size_GIGA_v:2\n");	
    rename(TMP_LOG_FILE, file);	
	GIGA_v(mutex2);
//    printf("syslog:cheak_file_size_GIGA_e:2\n");	
	}
	GBT_EXIT();
	return 0;
}
void GBT_count_init()
{
	if( (fire_count_shmid=
		shmget(FIRE_KEY_ID,sizeof(int),IPC_CREAT|0666))==-1)
		printf("fire_count_shmid --shmget error\n");
	if( (conn_count_shmid=
		shmget(CONN_KEY_ID,sizeof(int),IPC_CREAT|0666))==-1)
		printf("conn_count_shmid --shmget error\n");
	if( (upnp_count_shmid=
		shmget(UPNP_KEY_ID,sizeof(int),IPC_CREAT|0666))==-1)
		printf("upnp_count_shmid --shmget error\n");
	if( (fire_count=shmat(fire_count_shmid,NULL,0)) ==NULL)
		printf("fire_count --shmat error\n");
	if( (conn_count=shmat(conn_count_shmid,NULL,0)) ==NULL)
		printf("conn_count --shmat error\n");
	if( (upnp_count=shmat(upnp_count_shmid,NULL,0)) ==NULL)
		printf("upnp_count --shmat error\n");
	*fire_count=0;
	*conn_count=0;
	*upnp_count=0;


}
void GBT_count_cleanup(void){
	printf("Exiting Syslogd!\n");

	if (fire_count_shmid != -1)
		shmdt(fire_count);
	if (fire_count_shmid != -1)
		shmctl(fire_count_shmid, IPC_RMID, NULL);

	if (conn_count_shmid != -1)
		shmdt(conn_count);
	if (conn_count_shmid != -1)
		shmctl(conn_count_shmid, IPC_RMID, NULL);

	if (upnp_count_shmid != -1)
		shmdt(upnp_count);
	if (upnp_count_shmid != -1)
		shmctl(upnp_count_shmid, IPC_RMID, NULL);
}


/* << spark for log */


