#!/usr/bin/perl

# Size of mtd2 partition. To figure out this walue do
#
#   # cat /proc/mtd
#
# on your target system!
# For example, CNU-550pro gives:
#
# dev:    size   erasesize  name
# mtd0: 00010000 00002000 "bios"
# mtd1: 00010000 00010000 "nvram"
# mtd2: 00340000 00010000 "cramfs"
# mtd3: 000a0000 00010000 "repair"
#
# Figure out what partition is for cramfs and set MAX_CRAMFS_SIZE

$MAX_CRAMFS_SIZE = 0x340000;

sub usage
{
    print STDERR "usage: $0 [ -b image.bin ] [ image.cramfs ]\n";
    exit 1;
}

while ($_ = shift @ARGV) {
    if (/^-/) {
	$next = shift @ARGV;
	&usage unless defined $next;
	if ($_ eq "-b") { $BIN = $next; }
	else { &usage; }
	next;
    }
    $arg++;
    if ($arg == 1) {
	$CRAMFS = $_;
    }
    else {
	&usage;
    }
}

print STDERR "Creating firmware image".(defined $BIN ? " $BIN" : "").":\n";

open(F,$CRAMFS) || die "$CRAMFS: $!";
$size = (stat F)[7];
sysread(F,$image,$size) == $size || die "$CRAMFS: $!";
close F || die "$CRAMFS: $!";

print STDERR "  CRAMFS:     $CRAMFS($size)\n";

if ($size > $MAX_CRAMFS_SIZE)
{
    print STDERR "  CRAMFS:     Image too large!\n";
    exit;
}

# header:
$header = pack("U", 0x49);
$header .= pack("U", 0x54);
$header .= pack("U", 0x00);
$header .= pack("U", 0x01);
$header .= pack("N", 0x01234567);
$header .= pack("N", 0x01234567);
$header .= pack("N",$size);

$image = $header.$image;

print STDERR "BIN image: ".length($image)." bytes\n";

if (!defined $BIN) {
    print $image || die "(stdout): $!";
}
else {
    open(FILE,">$BIN") || die "create $BIN: $!";
    print FILE $image || die "write $BIN: $!";
    close FILE || die "close $BIN: $!";
}
